(* Custom build file created on Wed Nov  4 11:03:56 GMT 2009 *)
(* Process algebra is: ccs *)
(* Modules requested: testing divergence simulation agentextra pg eq contraction logic  *)
val optionalModules = "AgentExtra,Graph,Divergence,Contraction,\nEquivalences,Logic,Simulation,Testing"
(* Build file fragment: basic *)
(********************* BEGIN PERVASIVES ************************************)
fun profileStart _ = () (* relics from a bygone age *)
fun profileStop _ = ()
val date = Date.toString (Date.fromTimeLocal (Time.now ()))

(* If you want to use your pre-installed copies of the NJML library    *)
(* files, for example, so that you can save disk space by deleting the *)
(* copies that come with the CWB, you need to change the value of this *)
(* variable appropriately. But disk space is cheap now! *)
val njmlLibDir = "smlnj-lib/Util/";
  
use "common.ml";			(* creates structure Lib *)
use "SortedList.sig";
use "SortedList.str";
structure SL = SL();

use (njmlLibDir^"hash-string.sml");
use (njmlLibDir^"hash-key-sig.sml");
use (njmlLibDir^"hash-table-sig.sml");
use (njmlLibDir^"hash-table-rep.sml");	(* new in 110 *)
use (njmlLibDir^"hash-table.sml");				       
use (njmlLibDir^"lib-base-sig.sml");
use (njmlLibDir^"lib-base.sml");
use (njmlLibDir^"ord-key-sig.sml");
use (njmlLibDir^"ord-set-sig.sml");
use (njmlLibDir^"ord-map-sig.sml");
use (njmlLibDir^"binary-map-fn.sml");
use (njmlLibDir^"binary-set-fn.sml");
use (njmlLibDir^"atom-sig.sml");
use (njmlLibDir^"atom.sml");

(* doing away with a special small dict for now -- see how it goes...*)
functor SmallDict = BinaryMapFn;
  
use "Var.sig";
use "Var.str";
structure V = Var ();

use "Env.sig";
use "Env.str";
structure E = Env();			(* suitable for big var dicts *)

use "MaybeVar.sig";
use "MaybeVar.str";
structure MaybeVar = MaybeVar();

(* For when Env is too much overhead: *)
structure SmallDictKey : ORD_KEY =
  struct
    type ord_key = V.var
    val compare = V.compare
  end;
structure SmallVarDict = SmallDict (SmallDictKey);

use "OrderedSetElement.sig";
use "OrderedSet.sig";
use "OrderedSet.str";

(* Used to have datatypes for modules and PAs. For now, using strings: but *)
(* might possibly want to go back, hence all these id. functions *)
(* Using explicit strings in the modules is probably best regarded as a temp *)
(* measure, for experimental modules/PAs. For established ones, we define   *)
(* vals here. Or is this too polluting of the global namespace? *)
type cwbModule = string
val Basic = "Basic"
val Registry = "Registry"
val Contraction = "Contraction"
val DivPre = "DivPre" 
val Equiv = "Equiv"
val EquivalenceGames = "EquivalenceGames"
val Logic = "Logic"
val PABasic = "PABasic"
val PAExtra = "PAExtra"
val Sim = "Sim"
val TestOps = "TestOps"
val VPCCS = "VPCCS"

exception NotAModule
val cwbModuleString = id
val stringToM = id

type cwbProcessAlgebra = string
val CCS = "CCS"
val SCCS = "SCCS"
val NotApplicable = "NotApplicable" 

val cwbProcessAlgebraString = id
exception NotAPA
val stringToPA = id

val initialProcessAlgebra = ""; 
(* each PA had better UI.setCurrentProcessAlgebra before it tries to  *)
(* register any commands *)
  
use "DynamicCommands.sig";
use "DynamicCommands.str";	      (* refers to initialProcessAlgebra *)
use "UI.sig";
use "UI.str";

structure DynamicCommands = DynamicCommands();
structure UI = UI(structure DynamicCommands = DynamicCommands);

use "Tokens.sig";
use "Tokens.str";
use "cwb.lex.sml";			(* make structure CWBLex *)
use "logic/logic.lex.sml";		(* make structure PropLex *)
use "Lexer.sig";
use "Lexing.sig";
use "Lexing.str";	

(* Global Lexing/Parser, walks along commands. *)
structure Lexing = Lexing(structure Lexer = CWBLex);

(* provides save clear print *)
use "Registry.sig";
use "Registry.str";
structure Registry = Registry();

(* provides help cwb input output quit echo toggle *)
use "BasicCommands.str";
(* If &^$% ML would let me go structure _ = ..., I would. But it won't. *)
structure Foo = BasicCommands();

use "Top.sig";
use "Top.str";
structure Top = Top();

val cwb = Top.toploop;

val commonAgentDir = "agent/";
val agentDir = ref "" : string ref;

(* Build file fragment: ccs *)
agentDir := commonAgentDir^"ccs/";
UI.setCurrentProcessAlgebra CCS;

(* this part is really defunct, we're going to do values differently. *)

val valuesDir = "values/";
use (valuesDir^"Value.sig");
use (valuesDir^"ValueProtected.sig");
use (valuesDir^"NoValue.str");
use (valuesDir^"Eval.sig");
use (valuesDir^"Eval.str");
structure NoValue = NoValue();

(* end defunct *)

use (commonAgentDir^"Act.sig");
use ((!agentDir)^"ActProtected.sig");
use ((!agentDir)^"Act.str");
use ((!agentDir)^"Prefix.sig");
use ((!agentDir)^"Prefix.str");
use (commonAgentDir^"Agent.sig");
use ((!agentDir)^"AgentProtected.sig");
use ((!agentDir)^"Agent.str");

structure A = Act(NoValue);

structure Ag = Agent(structure A=A);

(* for backwards compatibility! *)
signature AGENT_WRAPPER = AGENT;
structure AgentWrapper = Ag;	


(* Build file fragment: agentextra *)
(* TCCS only: you can build this for SCCS but it's empty! *)
(* (Should organise this better really... *)
(* Provides: normalform init stable freevars sort states statesexp statesobs *)
(* deadlocks deadlocksobs findinit findinitobs random closure obs vs *)
(* derivatives *)
use ((!agentDir)^"AgentExtra.str");

structure Foo = AgentExtra (structure Ag = Ag);

(* Build file fragment: pg *)
(* provides "graph" "size" *)
use "PolyGraph.sig";
use "PolyGraph.str";
structure PG = PolyGraph(structure Ag = AgentWrapper);
 

(* Build file fragment: genp *)
(* require pg *)
(* provides nothing *)
(* this lot is all very incestuous and needs sorted... *)
use "Trie.sig";
use "Trie.str";
use "AccSet.sig";
use "AccSet.str";
use "TGraph.sig";
use "TGraph.str";
use "Elem.sig";
use "Elem.str";
use "DivComp.sig";
use "DivComp.str";
use "GenPre.sig";
use "GenPre.str";
structure Trie = Trie();
structure AS = AccSet(structure A = AgentWrapper.A 
		      structure Trie = Trie); 
structure TG = TGraph(structure PG = PG 
		      structure Trie = Trie 
		      structure AS = AS ); 
structure Elem = Elem(structure PG = PG 
		      structure AS = AS); 
structure DivC = DivComp(structure Elem = Elem); 
structure GenP = GenPre(structure DivC = DivC); 
(* ... ending here, well, just about. *)

(* Build file fragment: divergence *)
(* require genp *)
(* provides "twothirdspre" "twothirdseq" "pre" "precong" "strongpre" *)
(* requires GenP : GENPRE Ag : AGENT_WRAPPER *)
use "DivPre.str";
structure Foo = DivPre(structure GenP = GenP
		     structure Ag = AgentWrapper);
  

(* Build file fragment: contraction *)
(* require pg *)  
(* provides "contraction" *)
use "Contract.str";
structure Foo = Contract(structure PG = PG);

(* Build file fragment: eq *)
(* require pg *)
(* provides "min" "pb" "branchingeq" "diveq" "strongeq" "eq" "cong" *)
(* requires PG : POLYGRAPH *)
use "Equiv.sig";
use "Equiv.str";
structure Eq = Equiv(structure PG = PG);

(* Build file fragment: logic *)
(* require genp *)
val logicDir = "logic/";
  
  use (logicDir^"Modality.sig");
  use (logicDir^"ModalityProtected.sig");
  use (logicDir^"Modality.str");

  use (logicDir^"ModalityComplex.sig");
  use (logicDir^"ModalityComplexProtected.sig");
  use (logicDir^"ModalityComplex.str");

  use (logicDir^"TranslateModality.sig");
  use (logicDir^"TranslateModality.str");

  use (logicDir^"Logic.sig");

  use (logicDir^"LogicProtected.sig");
  use (logicDir^"Logic.str");

  use (logicDir^"Logic2Protected.sig");
  use (logicDir^"Logic2.str");
(* require genp *)
  use (logicDir^"MuCalculus.sig");
  use (logicDir^"MuCalculusProtected.sig");
  use (logicDir^"MuCalculus.str");

  use (logicDir^"ParseLogic.sig");
  use (logicDir^"ParseLogic.str");

  use (logicDir^"LogicParser.sig");
  use (logicDir^"LogicParser.str");

  use (logicDir^"MuCalculusFuns.sig");
  use (logicDir^"MuCalculusFuns.str");
  
  use (logicDir^"TranslateLogic.sig");
  use (logicDir^"TranslateLogic.str");
  use (logicDir^"TranslateLogic2.str");

(*   this depends on AccSet and TGraph -- does anything else? No. *)
  use (logicDir^"Df.sig");
  use (logicDir^"Df.str");

  use (logicDir^"HMLCheck.sig");
  use (logicDir^"HMLCheck.str");

  use (logicDir^"HMLSat.sig");
  use (logicDir^"HMLSat.str");

  use (logicDir^"GameCheck.sig");
  use (logicDir^"GameCheck.str");
  
  use ("LogicWrapper.str");

structure LogicWrapper =
      LogicWrapper (structure PG = PG);

(* Build file fragment: simulation *)
(* provides sim, and of course its subcommands *)
use "Sim.str";
structure Foo = Simulate(structure Ag = AgentWrapper);



(* Build file fragment: testing *)
(* requires: eq genp *)
(* provides "mayeq" "musteq" "testeq" "maypre" "mustpre" "testpre" *)
(* requires Eq : EQUIV TG : TGRAPH GenP : GENPRE Ag : AGENT_WRAPPER *)
(* Perdita, actually some parts depend on Eq and others on GenP. Split?? *)
use "TestOps.str";
structure Foo = TestOps(structure Eq = Eq 
		      structure TG = TG 
		      structure GenP = GenP
		      structure PG = PG); 

(* Build file fragment: end *)
fun exportedfun _ =
  (cwb();
   OS.Process.success)
  handle Error s => (print s; OS.Process.failure);
  
SMLofNJ.exportFn ("cwb", exportedfun);

(* End *)